@@ -34,9 +34,7 @@ class _State:
         self.write_err: Writer = _write_err
         self.installed: bool = False
         self.formatter: Formatter = base_formatters.repl
-        self.saved_info: List[Info] = []
-        self.friendly_info: List[core.FriendlyTraceback] = []
-        # TODO: is having both saved_info and friendly_info redundant?
+        self.recorded_tracebacks: List[core.FriendlyTraceback] = []
         self.include: InclusionChoice = "explain"
         self.lang: str = "en"
         self.install_gettext(self.lang)
@@ -60,7 +58,12 @@ class _State:
         self.is_jupyter: bool = False
         self.jupyter_button_style: str = ""
 
-    def show_traceback_info_again(self) -> None:
+        self.old_excepthook = None
+        # Include chain exception info in shortened/friendly traceback
+        # this should be disabled by Friendly itself.
+        self.include_chained_exception = True
+
+    def show_traceback_info_again(self, index: int = -1) -> None:
         """If has not been cleared, write the traceback info again, using
         the default stream.
 
@@ -68,10 +71,33 @@ class _State:
         level and wishes to see a traceback reexplained without having
         to execute the code again.
         """
-        if not self.saved_info:
+        if not self.recorded_tracebacks:
             print(_("Nothing to show: no exception recorded."))
             return
-        explanation = self.formatter(self.saved_info[-1], include=self.include)
+        pos_integer = _("`index` must be a positive integer or -1.\n")
+        if index != -1:
+            try:
+                if index != abs(int(index)) and index != 0:
+                    print(pos_integer)
+                    return
+            except Exception:
+                print(pos_integer)
+                return
+            if len(self.recorded_tracebacks) < index:
+                print(
+                    _("There are only {number} recorded exceptions.\n").format(
+                        number=len(self.recorded_tracebacks)
+                    )
+                )
+                return
+            index -= 1  # list index starts at zero
+
+        info = self.recorded_tracebacks[index].info
+        if info["lang"] != self.lang:
+            self.recorded_tracebacks[index].recompile_info()
+            info = self.recorded_tracebacks[index].info
+
+        explanation = self.formatter(info, include=self.include)
         self.write_err(explanation)
         # Do not combine with above as 'explanation' could be a list for IDLE
         self.write_err("\n")
@@ -96,13 +122,10 @@ class _State:
             return
         current_lang.install(lang)
         self.lang = lang
-        if self.saved_info:
-            if not self.friendly_info:  # pragma: no cover
-                debug_helper.log(
-                    "Problem: saved_info includes content but friendly doesn't."
-                )
-            self.friendly_info[-1].recompile_info()
-            self.friendly_info[-1].info["lang"] = lang
+        # TODO: this might not be needed, we could just update the language
+        # TODO: when display is needed.
+        if self.recorded_tracebacks:
+            self.recorded_tracebacks[-1].recompile_info()
 
     def install_gettext(self, lang: str) -> None:
         """Sets the current language for gettext."""
@@ -135,7 +158,7 @@ class _State:
         self,
         lang: Optional[str] = None,
         redirect: Union[str, Writer, None] = None,
-        include: InclusionChoice = "explain",
+        include: InclusionChoice = None,
     ) -> None:
         """Replaces sys.excepthook by friendly's own version."""
 
@@ -143,16 +166,21 @@ class _State:
             self.install_gettext(lang)
         if redirect is not None:
             self.set_redirect(redirect=redirect)
-        if include != self.include:
+        if include is not None:
             self.set_include(include)
+        if self.installed:
+            return
 
+        self.old_excepthook = sys.excepthook
         sys.excepthook = self.exception_hook
         self.installed = True
 
     def uninstall(self) -> None:
-        """Resets sys.excepthook to the Python default."""
+        """Resets sys.excepthook to the excepthook used before installing friendly-traceback."""
+        if not self.installed:
+            return
+        sys.excepthook = self.old_excepthook
         self.installed = False
-        sys.excepthook = sys.__excepthook__
 
     def set_redirect(self, redirect: Union[str, Writer, None] = None) -> None:
         """Sets where the output is redirected."""
@@ -189,11 +217,11 @@ class _State:
                 )
                 self.sys_last_type = sys.last_type
                 self.sys_last_value = sys.last_value
-            info = self.saved_info[-1]
+            info = self.recorded_tracebacks[-1].info
             self.output_info(info)
             return
-        else:
-            self.exception_before_import = False
+
+        self.exception_before_import = False
         self.exception_hook(etype, value, tb, redirect=redirect)
 
     def exception_hook(
@@ -269,11 +297,9 @@ class _State:
         Returns a dict containing the available info.
         """
         try:
-            self.friendly_info.append(core.FriendlyTraceback(etype, value, tb))
-            self.friendly_info[-1].compile_info()
-            info = self.friendly_info[-1].info
-            info["lang"] = self.lang
-            self.saved_info.append(info)
+            self.recorded_tracebacks.append(core.FriendlyTraceback(etype, value, tb))
+            self.recorded_tracebacks[-1].compile_info()
+            info = self.recorded_tracebacks[-1].info
         except Exception:  # pragma: no cover
             if not debug_helper.DEBUG:
                 print(
