@@ -14,12 +14,12 @@ from typing import Any, Callable, Dict, List, Optional, Type, Union
 import friendly_traceback
 from friendly_traceback import __version__, base_formatters, debug_helper
 from friendly_traceback.config import session
-from friendly_traceback.core import TracebackData
 from friendly_traceback.ft_gettext import current_lang
 from friendly_traceback.functions_help import add_help_attribute
 from friendly_traceback.info_generic import get_generic_explanation
 from friendly_traceback.path_info import show_paths
 from friendly_traceback.syntax_errors.source_info import Statement
+from friendly_traceback.tb_data import TracebackData
 from friendly_traceback.typing_info import InclusionChoice, Site
 
 _ = current_lang.translate
@@ -35,60 +35,81 @@ def back() -> None:
     The intention is to allow recovering from a typo when trying interactively
     to find out specific information about a given exception.
     """
-    if not session.saved_info:
-        session.write_err(_("Nothing to go back to: no exception recorded.") + "\n")
+    if not session.recorded_tracebacks:
+        info = {"message": _("Nothing to go back to: no exception recorded.") + "\n"}
+        explanation = session.formatter(info, include="message")
+        session.write_err(explanation)
         return
-    if not session.friendly_info:  # pragma: no cover
-        debug_helper.log("Problem: saved info is not empty but friendly is")
-    session.saved_info.pop()
-    session.friendly_info.pop()
-    if session.saved_info:
-        info = session.saved_info[-1]
+    session.recorded_tracebacks.pop()
+    if session.recorded_tracebacks:
+        info = session.recorded_tracebacks[-1].info
         if info["lang"] != friendly_traceback.get_lang():
             info["lang"] = friendly_traceback.get_lang()
-            session.friendly_info[-1].recompile_info()
+            session.recorded_tracebacks[-1].recompile_info()
 
 
-def explain(include: InclusionChoice = "explain") -> None:
+def disable():
+    """Disable friendly-traceback exception hook"""
+    if not session.installed:
+        info = {"message": _("Friendly-traceback is already disabled.") + "\n"}
+        explanation = session.formatter(info, include="message")
+        session.write_err(explanation)
+        return
+    session.uninstall()
+
+
+def enable():
+    """Enable friendly-traceback exception hook"""
+    if session.installed:
+        info = {"message": _("Friendly-traceback is already enabled.") + "\n"}
+        explanation = session.formatter(info, include="message")
+        session.write_err(explanation)
+        return
+    session.install()
+
+
+def explain(index: int = -1, include: InclusionChoice = "explain") -> None:
     """Shows the previously recorded traceback info again,
     with the option to specify different items to include.
     For example, ``explain("why")`` is equivalent to ``why()``.
     """
     old_include = friendly_traceback.get_include()
     friendly_traceback.set_include(include)
-    session.show_traceback_info_again()
+    session.show_traceback_info_again(index)
     friendly_traceback.set_include(old_include)
 
 
-def friendly_tb() -> None:
+def friendly_tb(index: int = -1) -> None:
     """Shows the simplified Python traceback,
     which includes the hint/suggestion if available.
     """
-    explain("friendly_tb")
+    explain(index, include="friendly_tb")
 
 
-def hint() -> None:
+def hint(index: int = -1) -> None:
     """Shows hint/suggestion if available."""
-    explain("hint")
+    explain(index, include="hint")
 
 
 def history() -> None:
     """Prints the list of error messages recorded so far."""
-    if not session.saved_info:
-        session.write_err(_nothing_to_show() + "\n")
+    if not session.recorded_tracebacks:
+        info = {"message": _nothing_to_show() + "\n"}
+        explanation = session.formatter(info, include="message")
+        session.write_err(explanation)
         return
-    for info in session.saved_info:
-        message = session.formatter(info, include="message")
-        if message:
-            message = message.replace("\n", "")
-            session.write_err(message)
+    for index, tb in enumerate(session.recorded_tracebacks):
+        if "message" in tb.info:
+            info = {"message": f"{index+1}. {tb.info['message']}"}
+            explanation = session.formatter(info, include="message")
+            session.write_err(explanation)
 
 
-def python_tb() -> None:
+def python_tb(index: int = -1) -> None:
     """Shows the Python traceback, excluding files from friendly
     itself.
     """
-    explain("python_tb")
+    explain(index, include="python_tb")
 
 
 def toggle_prompt() -> None:
@@ -97,7 +118,9 @@ def toggle_prompt() -> None:
 
 
 def what(
-    exception: Union[Type[BaseException], str, bytes, types.CodeType, None] = None,
+    exception_or_index: Union[
+        Type[BaseException], str, bytes, types.CodeType, int, None
+    ] = None,
     pre: bool = False,
 ) -> None:
     """If known, shows the generic explanation about a given exception.
@@ -106,45 +129,68 @@ def what(
     formatted in a way that is suitable for inclusion in the
     documentation.
     """
-    if exception is None:
-        explain("what")
+    if exception_or_index is None:
+        explain(index=-1, include="what")
+        return
+    elif isinstance(exception_or_index, int):
+        explain(index=exception_or_index, include="what")
         return
 
-    if inspect.isclass(exception) and issubclass(exception, BaseException):
-        result = get_generic_explanation(exception)
+    if inspect.isclass(exception_or_index) and issubclass(
+        exception_or_index, BaseException
+    ):
+        result = get_generic_explanation(exception_or_index)
     else:
         try:
-            exc = eval(exception)  # skipcq PYL-W0123
+            exc = eval(exception_or_index)  # skipcq PYL-W0123
             if inspect.isclass(exc) and issubclass(exc, BaseException):
                 result = get_generic_explanation(exc)
             else:
                 result = _("{exception} is not an exception.").format(
-                    exception=exception
+                    exception=f"`{exception_or_index}`"
                 )
         except Exception:  # noqa
-            result = _("{exception} is not an exception.").format(exception=exception)
+            result = _("{exception} is not an exception.").format(
+                exception=f"`{exception_or_index}`"
+            )
 
     if pre:  # for documentation # pragma: no cover
         lines = result.split("\n")
         for line in lines:
-            session.write_err("    " + line + "\n")
+            session.write_err(f"    {line}\n")
         session.write_err("\n")
     else:
-        session.write_err(result)
+        info = {"generic": result}
+        explanation = session.formatter(info, include="what")
+        session.write_err(explanation)
 
 
-def where() -> None:
+def where(index: int = -1, more=False) -> None:
     """Shows the information about where the exception occurred"""
-    explain("where")
+    if more:
+        try:
+            info = session.recorded_tracebacks[index].info
+        except Exception:
+            explain(index, "where")  # let the error bubble up
+        else:
+            if "detailed_tb" in info:
+                explain(index, "detailed_tb")
+            else:
+                explain(index, "where")
+    else:
+        explain(index, "where")
 
 
-def why() -> None:
+def why(index: int = -1) -> None:
     """Shows the likely cause of the exception."""
     # If no cause is found, and the exception name is not accompanied by a
     # message, as in "StopIteration:", we use the same info for
     # the cause as we used for the generic information as per issue #66
     try:
-        info = session.saved_info[-1]
+        if index == -1:
+            info = session.recorded_tracebacks[-1].info
+        else:
+            info = session.recorded_tracebacks[index + 1].info
     except IndexError:
         info = {}
     if (
@@ -153,9 +199,9 @@ def why() -> None:
         and len(info["message"].split(":")) > 1
         and not info["message"].split(":")[1].strip()  # empty message
     ):
-        explain("what")
+        explain(index, "what")
     else:
-        explain("why")
+        explain(index, "why")
 
 
 def www(site: Optional[Site] = None) -> None:  # pragma: no cover
@@ -211,8 +257,8 @@ def www(site: Optional[Site] = None) -> None:  # pragma: no cover
         )
         return
 
-    info = session.saved_info[-1] if session.saved_info else None
-    if site is None and info is not None:
+    info = session.recorded_tracebacks[-1].info if session.recorded_tracebacks else {}
+    if site is None and info:
         message = info["message"].replace("'", "")
         if " (" in message:
             message = message.split("(")[0]
@@ -251,18 +297,17 @@ def _debug_tb() -> None:  # pragma: no cover
     """Shows the true Python traceback, which includes
     files from friendly itself.
     """
-    explain("debug_tb")
+    explain(-1, include="debug_tb")
 
 
 def _get_exception() -> Optional[BaseException]:  # pragma: no cover
     """Debugging tool: returns the exception instance or None if no exception
     has been raised.
     """
-    if not session.saved_info:
+    if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None  # add explicit None here and elsewhere to silence mypy
-    info = session.saved_info[-1]
-    return info["_exc_instance"]
+    return session.recorded_tracebacks[-1].tb_data.exception_instance
 
 
 def _get_frame() -> Optional[types.FrameType]:  # pragma: no cover
@@ -270,11 +315,10 @@ def _get_frame() -> Optional[types.FrameType]:  # pragma: no cover
 
     This is not intended for end-users but is useful in development.
     """
-    if not session.saved_info:
+    if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None
-    info = session.saved_info[-1]
-    return info["_frame"]
+    return session.recorded_tracebacks[-1].tb_data.exception_frame
 
 
 def _get_statement() -> Optional[Statement]:  # pragma: no cover
@@ -284,11 +328,13 @@ def _get_statement() -> Optional[Statement]:  # pragma: no cover
 
     This is not intended for end-users but is useful in development.
     """
-    if not session.saved_info:
+    if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None
-    if isinstance(session.saved_info[-1]["_exc_instance"], SyntaxError):
-        return session.friendly_info[-1].tb_data.statement
+    if isinstance(
+        session.recorded_tracebacks[-1].tb_data.exception_instance, SyntaxError
+    ):
+        return session.recorded_tracebacks[-1].tb_data.statement
     print("No statement: not a SyntaxError.")
     return None
 
@@ -299,16 +345,15 @@ def _get_tb_data() -> Optional[TracebackData]:  # pragma: no cover
 
     This is not intended for end-users but is useful in development.
     """
-    if not session.saved_info:
+    if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None
-    info = session.saved_info[-1]
-    return info["_tb_data"]
+    return session.recorded_tracebacks[-1].tb_data
 
 
 def _get_info() -> list:
     """Debugging tool: returns the content of a traceback info."""
-    return session.saved_info[-1] if session.saved_info else []
+    return session.recorded_tracebacks[-1].info if session.recorded_tracebacks else []
 
 
 def _show_info() -> None:  # pragma: no cover
@@ -316,7 +361,7 @@ def _show_info() -> None:  # pragma: no cover
 
     Prints ``''`` for a given item if it is not present.
     """
-    info = session.saved_info[-1] if session.saved_info else []
+    info = session.recorded_tracebacks[-1].info if session.recorded_tracebacks else []
 
     for item in base_formatters.items_in_order:
         if item in info and info[item].strip():
@@ -327,13 +372,8 @@ def _show_info() -> None:  # pragma: no cover
         else:
             print(f"{item}: ''")
 
-    header_printed = False
     for item in info:
         if item not in base_formatters.items_in_order:
-            if not header_printed:
-                print("=" * 56)
-                print("The following are not meant to be shown to the end user:\n")
-                header_printed = True
             print(f"{item}: {info[item]}")
 
 
@@ -344,6 +384,8 @@ helpers: Dict[str, Callable[..., None]] = {
     "www": www,
     "explain": explain,
     "hint": hint,
+    "enable": enable,
+    "disable": disable,
     "back": back,
     "history": history,
     "friendly_tb": friendly_tb,
