@@ -1,8 +1,9 @@
 import os
+from typing import Iterable
 
 import stack_data
 from executing import only
-from stack_data import Line
+from stack_data import LINE_GAP, BlankLineRange, BlankLines, Formatter, Line, Options
 from stack_data.utils import cached_property
 
 from friendly_traceback import token_utils
@@ -14,6 +15,59 @@ from .source_cache import cache
 _ = current_lang.translate
 
 
+class FriendlyFormatter(Formatter):
+    def __init__(
+        self,
+        **kwargs,
+    ):
+        self.indent = "    "
+        super().__init__(**kwargs)
+
+    def format_frame_source(self, frame: stack_data.FrameInfo) -> Iterable[str]:
+        for line in frame.lines:
+            if isinstance(line, Line):
+                yield self.format_line(line)
+            elif isinstance(line, BlankLineRange):
+                yield self.format_blank_lines_linenumbers(line)
+            else:
+                assert line is LINE_GAP
+                yield self.indent + self.line_gap_string + "\n"
+
+    def format_line(self, line: Line) -> str:
+        result = self.indent
+        if line.is_current:
+            result += self.current_line_indicator
+        else:
+            result += " " * len(self.current_line_indicator)
+        result += self.line_number_format_string.format(line.lineno)
+        prefix = result
+        result += line.render() + "\n"
+
+        for line_range in line.executing_node_ranges:
+            start = line_range.start - line.leading_indent
+            end = line_range.end - line.leading_indent
+            # if end <= start, we have an empty line inside a highlighted
+            # block of code. In this case, we need to avoid inserting
+            # an extra blank line with no markers present.
+            if end > start:
+                result += (
+                    " " * (start + len(prefix))
+                    + self.executing_node_underline * (end - start)
+                    + "\n"
+                )
+        return result
+
+    def format_blank_lines_linenumbers(self, blank_line):
+        result = self.indent + " " * len(self.current_line_indicator)
+        if blank_line.begin_lineno == blank_line.end_lineno:
+            return (
+                result
+                + self.line_number_format_string.format(blank_line.begin_lineno)
+                + "\n"
+            )
+        return result + "{}\n".format(self.line_number_gap_string)
+
+
 class FrameInfo(stack_data.FrameInfo):
     @cached_property
     def partial_source(self):
@@ -31,7 +85,7 @@ class FrameInfo(stack_data.FrameInfo):
         file_not_found = _("Problem: source of `{filename}` is not available\n").format(
             filename=self.filename
         )
-        source = line = ""
+        source = ""
 
         if not self.lines and self.filename:
             # protecting against https://github.com/alexmojaki/stack_data/issues/13
@@ -48,7 +102,7 @@ class FrameInfo(stack_data.FrameInfo):
                 debug_helper.log_error(e)
 
         if self.lines:
-            source, line = self._highlighted_source(with_node_range)
+            source = self._highlighted_source(with_node_range)
         elif self.filename and os.path.abspath(self.filename):
             if self.filename not in ["<stdin>", "<string>"]:
                 # When filename is "<stdin>", "<string>",
@@ -70,7 +124,7 @@ class FrameInfo(stack_data.FrameInfo):
         if not source.endswith("\n"):
             source += "\n"
 
-        return {"source": source, "line": line}
+        return source
 
     @cached_property
     def highlighted_source(self):
@@ -82,6 +136,8 @@ class FrameInfo(stack_data.FrameInfo):
         for line_obj in self.lines:
             if line_obj is stack_data.LINE_GAP:
                 continue
+            elif isinstance(line_obj, stack_data.BlankLineRange):
+                continue
             elif line_obj.is_current:
                 return str(line_obj.text)
         return ""
@@ -106,111 +162,25 @@ class FrameInfo(stack_data.FrameInfo):
         if not lines:
             return "", ""
 
-        new_lines = []
-        problem_line = ""
         nb_digits = len(str(lines[-1].lineno))
-        # fmt: off
-        no_mark       = "       {:%d}: " % nb_digits  # noqa
-        with_mark     = "    -->{:%d}: " % nb_digits  # noqa
-        line_gap_mark = "      (...)"
-        # fmt: on
+        lineno_fmt_string = "{:%d}| " % nb_digits  # noqa
+        line_gap_string = " " * nb_digits + "(...)"
+        line_number_gap_string = " " * (nb_digits - 1) + ":"
 
-        def indent(begin):
-            return " " * (8 + nb_digits + begin + 1)
+        try:
+            new_lines = [
+                line
+                for line in FriendlyFormatter(
+                    options=Options(blank_lines=BlankLines.SINGLE),
+                    line_number_format_string=lineno_fmt_string,
+                    line_gap_string=line_gap_string,
+                    line_number_gap_string=line_number_gap_string,
+                ).format_frame_source(self)
+            ]
 
-        text_range_mark = None
-        if with_node_range and self.node_info:
-            if self.node_info[1]:
-                begin, end = self.node_info[1]
-                text_range_mark = indent(begin) + "^" * (end - begin)
-            elif self.node_info[2] and isinstance(self.node_info[2], str):
-                text = self.node_info[2]
-                for line_obj in lines:
-                    if line_obj is stack_data.LINE_GAP:
-                        continue
-                    if line_obj.is_current:
-                        begin = line_obj.text.find(text)
-                        if begin >= 0:
-                            text_range_mark = indent(begin) + "^" * len(text)
-
-        marked = False
-        prev_lineno = None
-        # stack_data does not include empty lines. However, I believe that
-        # this might not be helpful for beginners who look at the code
-        # without paying too much attention at the line numbers.
-        # I fixed this below.
-        indentations = []
-        for line_obj in lines:
-            if line_obj is stack_data.LINE_GAP:
-                new_lines.append(line_gap_mark)
-                prev_lineno = None
-                continue
-
-            if prev_lineno:
-                if line_obj.lineno - prev_lineno == 2:
-                    # add single empty line
-                    num = no_mark.format(prev_lineno + 1)
-                    new_lines.append(num)
-                elif line_obj.lineno - prev_lineno > 2:
-                    # add line gap to indicate that some empty lines were skipped
-                    new_lines.append(line_gap_mark)
-
-            if line_obj.is_current:
-                num = with_mark.format(line_obj.lineno)
-                problem_line = line_obj.text
-                new_lines.append(num + problem_line.rstrip())
-                if text_range_mark is not None:
-                    new_lines.append(text_range_mark)
-                    indentations.append(text_range_mark.count(" "))
-                elif self.node_info:
-                    try:
-                        for line_range in line_obj.executing_node_ranges:
-                            begin = line_range.start
-                            end = line_range.end
-                            text_range_mark = indent(begin) + "^" * (end - begin)
-                            new_lines.append(text_range_mark)
-                            indentations.append(text_range_mark.count(" "))
-                    except Exception:
-                        pass
-                marked = True
-            elif marked:
-                num = no_mark.format(line_obj.lineno)
-                new_lines.append(num + line_obj.text.rstrip())
-                for line_range in line_obj.executing_node_ranges:
-                    begin = len(line_obj.text) - len(line_obj.text.lstrip())
-                    end = line_range.end
-                    text_range_mark = indent(begin) + "^" * (end - begin)
-                    new_lines.append(text_range_mark)
-                    indentations.append(text_range_mark.count(" "))
-            else:
-                num = no_mark.format(line_obj.lineno)
-                new_lines.append(num + line_obj.text.rstrip())
-            prev_lineno = line_obj.lineno
-
-        def re_indent(new_lines, indentations):
-            """Reindent the marked lines so that they are aligned as a block
-            except perhaps for the first line."""
-            min_indent = min(indentations)
-            indentations.pop(0)
-            skipped_first = False
-            re_indented_lines = []
-            for line in new_lines:
-                if set(line).issubset({" ", "^"}) and skipped_first:
-                    current_indent = line.count(" ")
-                    line = (
-                        " " * min_indent
-                        + "^" * (current_indent - min_indent)
-                        + line.lstrip()
-                    )
-                elif set(line).issubset({" ", "^"}):
-                    skipped_first = True
-                re_indented_lines.append(line)
-            return re_indented_lines
-
-        if len(indentations) > 1:
-            new_lines = re_indent(new_lines, indentations)
-
-        return "\n".join(new_lines), problem_line
+            return "".join(new_lines)
+        except Exception:
+            return "<NO SOURCE>"
 
     @cached_property
     def node_info(self):
@@ -260,14 +230,6 @@ class FrameInfo(stack_data.FrameInfo):
                     return None
 
         node_range = None
-        if "\n" in node_text:
-            tokens = token_utils.tokenize(node_text)
-            tokens = [tok for tok in tokens if tok != "\n"]
-            node_text = "".join(tok.string for tok in tokens)
-            # node spans multiple lines; we only highlight
-            # part of a single line where a node is found.
-            return node, node_range, node_text
-
         bad_line = self.current_line.text
         if node_text and node_text in bad_line:
             begin = bad_line.find(node_text)
