@@ -5,9 +5,10 @@ can be useful for beginners without overwhelming them.
 """
 import ast
 import builtins
+import re
 import sys
 import types
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Union
 
 from . import debug_helper, token_utils, utils
 from .ft_gettext import current_lang
@@ -25,6 +26,44 @@ except ImportError:  # pragma: no cover
 INDENT = " " * 8
 MAX_LENGTH = 65
 _ = current_lang.translate
+RENAMED_BUILTINS = set()
+
+
+class ConfidentialInformation:
+    """Used to hide values of confidential information"""
+
+    words = []
+    regex = []
+    redacted = "••••••"
+
+    def hide_confidential_information(
+        self, words: Union[List, None] = None, patterns: Union[List, None] = None
+    ) -> None:
+        """Use to record words or regular expression patterns that determine
+        if a variable represents confidential information.
+        """
+        if words is not None:
+            self.words.extend(words)
+        if patterns is not None:
+            for pattern in patterns:
+                self.regex.append(re.compile(pattern))
+
+    def is_confidential(self, name: str) -> bool:
+        """Identify variable names that are deemed to contain confidential information"""
+        if name in self.words:
+            return True
+        for pattern in self.regex:
+            if re.fullmatch(pattern, name):
+                return True
+        return False
+
+    def redact_confidential(self, name: str, value: Any) -> Any:
+        if confidential.is_confidential(name):
+            return confidential.redacted
+        return value
+
+
+confidential = ConfidentialInformation()
 
 
 def convert_type(short_form: str) -> str:
@@ -52,7 +91,7 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
     answer to "where()" or they can be used during the analysis
     of the cause of the exception.
 
-    The dict returned has five keys.
+    The dict returned has six keys.
     The first three, 'locals', 'globals', 'builtins',
     each containing a list of tuples, each tuple being of the form
     (name, repr(obj), obj) where name --> obj.
@@ -67,6 +106,7 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
         "builtins": [],
         "expressions": [],
         "name, obj": [],
+        "name, type": [],
     }
 
     scopes = (
@@ -90,11 +130,19 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
                     obj = scope_dict[name]
                     if hasattr(obj, "true_repr"):  # sourcery: skip
                         # guard against the case where obj == Friendly; #106
-                        repr_obj = obj.true_repr()
+                        repr_obj = str(obj.true_repr())
+                        # wrapped in str() for added security in case someone
+                        # else uses an attribute called true_repr
                     else:
-                        repr_obj = repr(obj)
+                        try:
+                            repr_obj = repr(obj)
+                        except Exception:  # issue #161: repr not returning a string
+                            repr_obj = str(type(obj))
                     objects[scope].append((name, repr_obj, obj))
                     objects["name, obj"].append((name, obj))
+                    obj_type = type(obj).__name__
+                    if obj_type is not None:
+                        objects["name, type"].append((name, obj_type))
                     break
             else:
                 if name in dir(builtins):
@@ -102,6 +150,9 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
                     obj = getattr(builtins, name)
                     objects["builtins"].append((name, repr(obj), obj))
                     objects["name, obj"].append((name, obj))
+                    obj_type = type(obj).__name__
+                    if obj_type is not None:
+                        objects["name, type"].append((name, obj_type))
 
     line = line.strip()
     if line.startswith(("def", "if", "while", "class", "for")) and line.endswith(":"):
@@ -109,9 +160,15 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
     try:
         atok = ASTTokens(line.strip(), parse=True)
     except SyntaxError as e:
-        if "unexpected EOF" not in str(e):
-            debug_helper.log(f"Problem with ASTTokens: {e}" + f"\nline: {line}")
-        return objects
+        if "unexpected EOF" in str(e):
+            return objects
+        if "\n" in line:
+            newline = " ".join(line.split())
+            try:
+                atok = ASTTokens(newline.strip(), parse=True)
+            except SyntaxError as e:
+                debug_helper.log(f"Problem with ASTTokens: {e}" + f"\nline: {line}")
+                return objects
 
     if atok is not None:
         evaluator = Evaluator.from_frame(frame)
@@ -217,7 +274,7 @@ def get_definition_scope(variable_name: str, frame: types.FrameType) -> List[Sco
     return scopes
 
 
-def get_var_info(line: str, frame: types.FrameType) -> str:
+def get_var_info(line: str, frame: types.FrameType) -> dict:
     """Given a frame object, it obtains the value (repr) of the names
     found in the logical line (which may span many lines in the file)
     where the exception occurred.
@@ -251,7 +308,33 @@ def get_var_info(line: str, frame: types.FrameType) -> str:
 
     if names_info:
         names_info.append("")
-    return "\n".join(names_info)
+    var_info = {"var_info": "\n".join(names_info)}
+    builtins_warnings = find_renamed_builtins(frame)
+    if builtins_warnings:
+        var_info["additional variable warning"] = builtins_warnings
+    return var_info
+
+
+def find_renamed_builtins(frame) -> str:
+    warnings = ""
+    for name in dir(builtins):
+        if name.startswith("_"):
+            continue
+        if name in frame.f_locals:
+            builtin_obj = getattr(builtins, name)
+            obj = frame.f_locals[name]
+            if builtin_obj != obj:
+                if name not in RENAMED_BUILTINS:  # only once
+                    warnings += _(
+                        "Friendly warning: you have redefined the python builtin `{name}`.\n"
+                    ).format(name=name)
+                    RENAMED_BUILTINS.add(name)
+            elif name in RENAMED_BUILTINS:
+                RENAMED_BUILTINS.remove(name)
+        elif name in RENAMED_BUILTINS:
+            RENAMED_BUILTINS.remove(name)
+
+    return warnings
 
 
 def simplify_repr(name: str, splitlines: bool = True) -> str:
@@ -360,46 +443,62 @@ def format_var_info(name: str, value: str, obj: str, _global: str = "") -> str:
     if _global:
         _global = "global "
 
-    not_repr = True
+    value = confidential.redact_confidential(name, value)
+
     if value.startswith("<") and value.endswith(">"):
         value = simplify_repr(value)
-        not_repr = False
-
-    if len(value) > MAX_LENGTH and not_repr:
-        # We reduce the length of the repr, indicate this by ..., but we
-        # also keep the last character so that the repr of a list still
-        # ends with ], that of a tuple still ends with ), etc.
-        if "," in value:  # try to truncate at a natural place
-            parts = value.split(", ")
-            length = 0
-            new_parts = []
-            for part in parts:
-                if len(part) + length > MAX_LENGTH:
-                    break
-                new_parts.append(part + ", ")
-                length += len(part) + 2
-            if new_parts:
-                value = "".join(new_parts) + "..." + value[-1]
-            else:
-                value = value[0 : MAX_LENGTH - 5] + "..." + value[-1]
-        else:
-            value = value[0 : MAX_LENGTH - 5] + "..." + value[-1]
-        try:
-            length_info = len(obj)
-        except OverflowError:
-            length_info = _("Object too large to be processed by Python.")
-        except TypeError:
-            pass
-        except Exception as e:
-            length_info = _("Unable to compute.") + f" ({e.__class__.__name__})"
+    elif "\n" in value:
+        value = format_multiline(value)
+    elif len(value) > MAX_LENGTH:
+        value, length_info = shorten_long_line(value, obj)
 
     result = f"    {_global}{name}:  {value}"
     if length_info:
-        _indent = " " * (7 + len(name))
-        result += f"\n{_indent}len({name}): {length_info}\n"
+        indent = " " * min(7 + len(name), 12)
+        result += f"\n{indent}len({name}): {length_info}\n"
     return result
 
 
+def format_multiline(value: str) -> str:
+    # This is useful for tabular data
+    indent = "\n" + " " * 8
+    lines = value.split("\n")
+    new_lines = [line if len(line) < 72 else line[:68] + "..." for line in lines]
+    if len(new_lines) > 6:
+        new_lines = new_lines[:6] + ["..."]
+    return indent + indent.join(new_lines)
+
+
+def shorten_long_line(value: str, obj: str) -> (str, str):
+    # We reduce the length of the repr, indicate this by ..., but we
+    # also keep the last character so that the repr of a list still
+    # ends with ], that of a tuple still ends with ), etc.
+    if "," in value:  # try to truncate at a natural place
+        parts = value.split(", ")
+        length = 0
+        new_parts = []
+        for part in parts:
+            if len(part) + length > MAX_LENGTH:
+                break
+            new_parts.append(part + ", ")
+            length += len(part) + 2
+        if new_parts:
+            value = "".join(new_parts) + "..." + value[-1]
+        else:
+            value = value[0 : MAX_LENGTH - 5] + "..." + value[-1]
+    else:
+        value = value[0 : MAX_LENGTH - 5] + "..." + value[-1]
+    try:
+        length_info = len(obj)
+    except OverflowError:
+        length_info = _("Object too large to be processed by Python.")
+    except TypeError:
+        length_info = ""
+    except Exception as e:
+        length_info = _("Unable to compute.") + f" ({e.__class__.__name__})"
+    return value, length_info
+
+
 def get_similar_names(name: str, frame: types.FrameType) -> SimilarNamesInfo:
     """This function looks for objects with names similar to 'name' in
     either the current locals() and globals() as well as in
