@@ -10,12 +10,13 @@ from types import FrameType
 from typing import Any, List, Optional, Tuple, Type
 
 from .. import debug_helper, info_variables, token_utils, utils
-from ..core import TracebackData
-from ..ft_gettext import current_lang, no_information
-from ..typing_info import CauseInfo
+from ..ft_gettext import current_lang, no_information, please_report
+from ..message_parser import get_parser
+from ..tb_data import TracebackData  # for type checking only
+from ..typing_info import CauseInfo  # for type checking only
 
 convert_type = info_variables.convert_type
-parser = utils.RuntimeMessageParser()
+parser = get_parser(TypeError)
 _ = current_lang.translate
 
 
@@ -61,9 +62,9 @@ def _convert_str_to_number(
     return cause, hint
 
 
-@parser.add
+@parser._add
 def cant_take_floor_or_mod_of_complex_number(
-    message: str, _frame: FrameType, tb_data: TracebackData
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
     if "can't take floor or mod of complex number." not in message:
         return {}
@@ -81,10 +82,8 @@ def cant_take_floor_or_mod_of_complex_number(
     return {"cause": cause}
 
 
-@parser.add
-def unsupported_type_for_divmod(
-    message: str, _frame: FrameType, _tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def unsupported_type_for_divmod(message: str, _tb_data: TracebackData) -> CauseInfo:
     # TODO: try with string arguments
     if "unsupported operand type(s) for divmod()" not in message:
         return {}
@@ -96,30 +95,36 @@ def unsupported_type_for_divmod(
     return {"cause": cause}
 
 
-@parser.add
-def getattr_attribute_name_must_be_string(
-    message: str, _frame: FrameType, _tb_data: TracebackData
+@parser._add
+def getattr_or_hasattr_attribute_name_must_be_string(
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
-    if "getattr(): attribute name must be string" not in message:
+    # Python 3.11 does not include 'getattr()' nor 'hasattr()' in message
+    # We make the assumption that if the name of these function appears in
+    # the bad_line, it is the source of the error.
+    if "attribute name must be string" not in message:
         return {}
-    cause = _("The second argument of the function `getattr()` must be a string.\n")
+    if (
+        "getattr(): attribute name must be string" in message
+        or "getattr" in tb_data.bad_line
+    ):
+        cause = _("The second argument of the function `getattr()` must be a string.\n")
+    elif (
+        "hasattr(): attribute name must be string" in message
+        or "hasattr" in tb_data.bad_line
+    ):
+        cause = _("The second argument of the function `hasattr()` must be a string.\n")
+    else:
+        cause = (
+            _("Some attribute of a function you called is expected to be a string.\n")
+            + please_report()
+        )
+
     return {"cause": cause}
 
 
-@parser.add
-def hasattr_attribute_name_must_be_string(
-    message: str, _frame: FrameType, _tb_data: TracebackData
-) -> CauseInfo:
-    if "hasattr(): attribute name must be string" not in message:
-        return {}
-    cause = _("The second argument of the function `hasattr()` must be a string.\n")
-    return {"cause": cause}
-
-
-@parser.add
-def parse_can_only_concatenate(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def parse_can_only_concatenate(message: str, tb_data: TracebackData) -> CauseInfo:
     # example: can only concatenate str (not "int") to str
     pattern = re.compile(
         r"can only concatenate (\w+) \(not [\'\"](\w+)[\'\"]\) to (\w+)"
@@ -128,8 +133,9 @@ def parse_can_only_concatenate(
     if match is None:
         return {}
 
-    obj_type1 = match.group(1)
-    obj_type2 = match.group(2)
+    obj_type1 = match[1]
+    obj_type2 = match[2]
+    frame = tb_data.exception_frame
 
     cause = _(
         "You tried to concatenate (add) two different types of objects:\n"
@@ -144,10 +150,8 @@ def parse_can_only_concatenate(
     return {"cause": cause}
 
 
-@parser.add
-def parse_must_be_str(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def parse_must_be_str(message: str, tb_data: TracebackData) -> CauseInfo:
     # python 3.6 version: must be str, not int
     # example: can only concatenate str (not "int") to str
     pattern = re.compile(r"must be str, not (\w+)")
@@ -155,23 +159,22 @@ def parse_must_be_str(
     if match is None:
         return {}
 
+    frame = tb_data.exception_frame
     cause = _(
         "You tried to concatenate (add) two different types of objects:\n"
         "{first} and {second}.\n"
-    ).format(first=convert_type("str"), second=convert_type(match.group(1)))
-    if match.group(1) in ["int", "float", "complex"]:
+    ).format(first=convert_type("str"), second=convert_type(match[1]))
+    if match[1] in ["int", "float", "complex"]:
         more_cause, possible_hint = _convert_str_to_number(
-            "str", match.group(1), frame, tb_data
+            "str", match[1], frame, tb_data
         )
         if more_cause is not None:
             return {"cause": cause + more_cause, "suggest": possible_hint}
     return {"cause": cause}
 
 
-@parser.add
-def parse_unsupported_operand_type(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def parse_unsupported_operand_type(message: str, tb_data: TracebackData) -> CauseInfo:
     more_cause = possible_hint = hint = None
     # example: unsupported operand type(s) for +: 'int' and 'str'
     pattern = re.compile(
@@ -181,10 +184,11 @@ def parse_unsupported_operand_type(
     if match is None:
         return {}
 
+    frame = tb_data.exception_frame
     all_objects = info_variables.get_all_objects(tb_data.bad_line, frame)["name, obj"]
-    operator = match.group(1)
-    obj_type1 = match.group(2)
-    obj_type2 = match.group(3)
+    operator = match[1]
+    obj_type1 = match[2]
+    obj_type2 = match[3]
     if operator in ["+", "+="]:
         cause = _(
             "You tried to add two incompatible types of objects:\n"
@@ -282,10 +286,8 @@ def parse_unsupported_operand_type(
     return cause
 
 
-@parser.add
-def parse_order_comparison(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def parse_order_comparison(message: str, tb_data: TracebackData) -> CauseInfo:
     # example: '<' not supported between instances of 'int' and 'str'
     pattern = re.compile(
         r"[\'\"](.+)[\'\"] not supported between instances of [\'\"](\w+)[\'\"] and [\'\"](\w+)[\'\"]"  # noqa
@@ -294,12 +296,13 @@ def parse_order_comparison(
     if match is None:
         return {}
 
-    if match.group(2) == match.group(3) == "complex":
+    frame = tb_data.exception_frame
+    if match[2] == match[3] == "complex":
         hint = _("Complex numbers cannot be ordered.\n")
         cause = _(
             "You tried to do an order comparison ({operator})\n"
             "between two complex numbers.\n"
-        ).format(operator=match.group(1))
+        ).format(operator=match[1])
         return {"cause": cause, "suggest": hint}
 
     cause = _(
@@ -307,16 +310,16 @@ def parse_order_comparison(
         "between two incompatible types of objects:\n"
         "{first} and {second}.\n"
     ).format(
-        operator=match.group(1),
-        first=convert_type(match.group(2)),
-        second=convert_type(match.group(3)),
+        operator=match[1],
+        first=convert_type(match[2]),
+        second=convert_type(match[3]),
     )
 
     number = None
-    if match.group(2) in ["int", "float"] and match.group(3) == "str":
-        number = match.group(2)
-    elif match.group(3) in ["int", "float"] and match.group(2) == "str":
-        number = match.group(3)
+    if match[2] in ["int", "float"] and match[3] == "str":
+        number = match[2]
+    elif match[3] in ["int", "float"] and match[2] == "str":
+        number = match[3]
 
     if number is not None:
         more_cause, possible_hint = _convert_str_to_number(
@@ -328,10 +331,8 @@ def parse_order_comparison(
     return {"cause": cause}
 
 
-@parser.add
-def bad_operand_type_for_unary(
-    message: str, _frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def bad_operand_type_for_unary(message: str, tb_data: TracebackData) -> CauseInfo:
     # example: bad operand type for unary +: 'str'
     pattern = re.compile(r"bad operand type for unary (.+): [\'\"](\w+)[\'\"]")
     match = re.search(pattern, message)
@@ -341,7 +342,7 @@ def bad_operand_type_for_unary(
     hint = None
     # The user might have written something like "=+" instead of
     # "+="
-    operator = match.group(1)
+    operator = match[1]
     index = token_utils.find_substring_index(
         tb_data.original_bad_line, tb_data.bad_line
     )
@@ -359,7 +360,7 @@ def bad_operand_type_for_unary(
         "You tried to use the unary operator '{operator}'\n"
         "with the following type of object: {obj}.\n"
         "This operation is not defined for this type of object.\n"
-    ).format(operator=operator, obj=convert_type(match.group(2)))
+    ).format(operator=operator, obj=convert_type(match[2]))
     if hint is not None:
         cause += "\n" + hint + "\n"
     cause = {"cause": cause}
@@ -369,9 +370,9 @@ def bad_operand_type_for_unary(
     return cause
 
 
-@parser.add
+@parser._add
 def does_not_support_item_assignment(
-    message: str, _frame: FrameType, _tb_data: TracebackData
+    message: str, _tb_data: TracebackData
 ) -> CauseInfo:
     # example: 'tuple' object does not support item assignment
     pattern = re.compile(r"[\'\"](\w+)[\'\"] object does not support item assignment")
@@ -380,7 +381,7 @@ def does_not_support_item_assignment(
         return {}
 
     hint = None
-    name = match.group(1)
+    name = match[1]
     cause = _(
         "In Python, some objects are known as immutable:\n"
         "once defined, their value cannot be changed.\n"
@@ -396,9 +397,9 @@ def does_not_support_item_assignment(
     return cause
 
 
-@parser.add
+@parser._add
 def exception_derived_from_base_exception(
-    message: str, _frame: FrameType, _tb_data: TracebackData
+    message: str, _tb_data: TracebackData
 ) -> CauseInfo:
     if "exceptions must derive from BaseException" in message:
         return {
@@ -407,9 +408,9 @@ def exception_derived_from_base_exception(
     return {}
 
 
-@parser.add
+@parser._add
 def incorrect_nb_positional_arguments(
-    message: str, _frame: FrameType, tb_data: TracebackData
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
     missing_self = False
     # example: my_function() takes 0 positional arguments but x was/were given
@@ -420,9 +421,9 @@ def incorrect_nb_positional_arguments(
         return {}
 
     hint = None
-    fn_name = match.group(1)[:-2]
-    nb_required = match.group(2)
-    nb_given = match.group(3)
+    fn_name = match[1][:-2]
+    nb_required = match[2]
+    nb_given = match[3]
     if ".<locals>." in fn_name:
         fn_name = fn_name.split(".<locals>.")[1]
     if int(nb_given) - int(nb_required) == 1:
@@ -454,10 +455,8 @@ def incorrect_nb_positional_arguments(
     return cause
 
 
-@parser.add
-def missing_positional_arguments(
-    message: str, _frame: FrameType, _tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def missing_positional_arguments(message: str, _tb_data: TracebackData) -> CauseInfo:
     # example: my_function() missing 1 required positional argument
     pattern = re.compile(r"(.*) missing (\d+) required positional argument")
     match = re.search(pattern, message)
@@ -469,20 +468,19 @@ def missing_positional_arguments(
         "cause": _(
             "You apparently have called the function '{fn_name}' with\n"
             "fewer positional arguments than it requires ({nb_required} missing).\n"
-        ).format(fn_name=match.group(1), nb_required=match.group(2))
+        ).format(fn_name=match[1], nb_required=match[2])
     }
 
 
-@parser.add
-def x_is_not_callable(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def x_is_not_callable(message: str, tb_data: TracebackData) -> CauseInfo:
     pattern = re.compile(r"'(.*)' object is not callable")
     match = re.search(pattern, message)
     if match is None:
         return {}
 
-    obj_type = match.group(1)
+    frame = tb_data.exception_frame
+    obj_type = match[1]
     if obj_type == "NoneType":
         none_type = _(
             "\nNote: `NoneType` means that the object has a value of `None`.\n"
@@ -586,13 +584,12 @@ def forgot_to_convert_name_to_int(name: str) -> Tuple[str, str]:
     return additional_cause, hint
 
 
-@parser.add
-def cannot_multiply_by_str(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def cannot_multiply_by_str(message: str, tb_data: TracebackData) -> CauseInfo:
     if "can't multiply sequence by non-int of type 'str'" not in message:
         return {}
 
+    frame = tb_data.exception_frame
     cause = _(
         "You can only multiply sequences, such as list, tuples,\n "
         "strings, etc., by integers.\n"
@@ -634,7 +631,7 @@ def find_possible_integers(
     for name, obj in all_objects["name, obj"]:
         if isinstance(obj, object_of_type):
             try:
-                int(obj)
+                int(obj)  # noqa
                 names.append(name)
             except Exception:  # noqa
                 pass
@@ -642,16 +639,17 @@ def find_possible_integers(
     return names
 
 
-@parser.add
+@parser._add
 def object_cannot_be_interpreted_as_an_integer(
-    message: str, frame: FrameType, tb_data: TracebackData
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
     pattern = re.compile(r"'(.*)' object cannot be interpreted as an integer")
     match = re.search(pattern, message)
     if match is None:
         return {}
 
-    obj_name = match.group(1)
+    frame = tb_data.exception_frame
+    obj_name = match[1]
     if obj_name == "NoneType":
         cause = _(
             "You wrote an object whose value is `None` where an integer was expected.\n"
@@ -687,17 +685,18 @@ def object_cannot_be_interpreted_as_an_integer(
     return cause
 
 
-@parser.add
+@parser._add
 def indices_must_be_integers_or_slices(
-    message: str, frame: FrameType, tb_data: TracebackData
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
     pattern = re.compile(r"(.*) indices must be integers or slices, not (.*)")
     match = re.search(pattern, message)
     if match is None:
         return {}
 
-    container_type = match.group(1)
-    index_type = match.group(2)
+    frame = tb_data.exception_frame
+    container_type = match[1]
+    index_type = match[2]
     cause = _(
         "In the expression `{line}`\n"
         "what is included between the square brackets, `[...]`,\n"
@@ -813,9 +812,9 @@ def indices_must_be_integers_or_slices(
     return {"cause": cause}
 
 
-@parser.add
+@parser._add
 def slice_indices_must_be_integers_or_none(
-    message: str, _frame: FrameType, _tb_data: TracebackData
+    message: str, _tb_data: TracebackData
 ) -> CauseInfo:
     if message != (
         "slice indices must be integers or None or have an __index__ method"
@@ -832,10 +831,8 @@ def slice_indices_must_be_integers_or_none(
     return {"cause": cause}
 
 
-@parser.add
-def unhashable_type(
-    message: str, _frame: FrameType, _tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def unhashable_type(message: str, _tb_data: TracebackData) -> CauseInfo:
     pattern = re.compile(r"unhashable type: '(.*)'")
     match = re.search(pattern, message)
     if match is None:
@@ -848,7 +845,7 @@ def unhashable_type(
         "once they have been created."
     )
 
-    original = match.group(1)
+    original = match[1]
     replacements = {"list": "tuple", "set": "frozenset"}
     if original in replacements:
         cause += _(
@@ -861,16 +858,15 @@ def unhashable_type(
     return {"cause": cause}
 
 
-@parser.add
-def object_is_not_subscriptable(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def object_is_not_subscriptable(message: str, tb_data: TracebackData) -> CauseInfo:
     pattern = re.compile(r"'(.*)' object is not subscriptable")
     match = re.search(pattern, message)
     if match is None:
         return {}
 
-    obj_type = match.group(1)
+    frame = tb_data.exception_frame
+    obj_type = match[1]
     if obj_type == "NoneType":
         none_type = _(
             "\nNote: `NoneType` means that the object has a value of `None`.\n"
@@ -886,7 +882,7 @@ def object_is_not_subscriptable(
     # first, try to identify object
     all_objects = info_variables.get_all_objects(tb_data.bad_line, frame)
     for name, obj in all_objects["name, obj"]:
-        truncated = tb_data.bad_line.replace(name, "", 1)
+        truncated = tb_data.bad_line.replace(name, "", 1).strip()
         if truncated.startswith("[") and truncated.endswith("]"):
             break
     else:
@@ -897,7 +893,15 @@ def object_is_not_subscriptable(
         return {"cause": cause + none_type}
 
     if callable(obj):
-        line = name + "(" + truncated[1:-1] + ")"
+        arg = truncated[1:-1]
+        if "," in arg:
+            # list[1, 2, 3] --> list((1, 2, 3))
+            try:
+                if len(inspect.getfullargspec(obj).args) == 1:
+                    arg = f"({arg})"
+            except:  # noqa
+                pass
+        line = f"{name}({arg})"
         hint = _("Did you mean `{line}`?\n").format(line=line)
         cause += "\n" + _("Perhaps you meant to write `{line}`.\n").format(line=line)
         return {"cause": cause, "suggest": hint}
@@ -910,10 +914,42 @@ def object_is_not_subscriptable(
     return {"cause": cause + none_type}
 
 
-@parser.add
-def object_is_not_iterable(
-    message: str, _frame: FrameType, _tb_data: TracebackData
+@parser._add
+def argument_of_object_is_not_iterable(
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
+    """This is usually the result of checking if something is contained
+    in an object, so the code would include '... in ...'."""
+    pattern = re.compile(r"argument of type '(.*)' is not iterable")
+    match = re.search(pattern, message)
+    if match is None:
+        return {}
+    obj_type = match[1]
+    # Suppose we have two objects of the same type, a and b.
+    # For the expression:
+    #    if a in b
+    # we want to identify 'b' and not 'a'.
+    if "in" in tb_data.bad_line:
+        after_in = tb_data.bad_line.split(" in ", 1)[1]
+    else:  # should never happen; see docstring
+        after_in = tb_data.bad_line
+    all_obj = info_variables.get_all_objects(after_in, tb_data.exception_frame)
+    for obj_name, obj_type2 in all_obj["name, type"]:
+        if obj_type2 == obj_type:
+            break
+    else:  # Using the info from the exception message
+        obj_name = obj_type
+
+    cause = _(
+        "An iterable is an object capable of returning its members one at a time.\n"
+        "Python containers (`list, tuple, dict`, etc.) are iterables.\n"
+        "'{obj_name}' is not a container. A container is required here.\n"
+    ).format(obj_name=obj_name)
+    return {"cause": cause}
+
+
+@parser._add
+def object_is_not_iterable(message: str, _tb_data: TracebackData) -> CauseInfo:
     pattern = re.compile(r"'(.*)' object is not iterable")
     match = re.search(pattern, message)
     if match is None:
@@ -927,10 +963,8 @@ def object_is_not_iterable(
     return {"cause": cause}
 
 
-@parser.add
-def cannot_unpack_non_iterable(
-    message: str, _frame: FrameType, _tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def cannot_unpack_non_iterable(message: str, _tb_data: TracebackData) -> CauseInfo:
     pattern = re.compile(r"cannot unpack non-iterable (.*) object")
     match = re.search(pattern, message)
     if match is None:
@@ -944,13 +978,13 @@ def cannot_unpack_non_iterable(
         "An iterable is an object capable of returning its members one at a time.\n"
         "Python containers (`list, tuple, dict`, etc.) are iterables,\n"
         "but not objects of type `{obj_type}`.\n"
-    ).format(obj_type=match.group(1))
+    ).format(obj_type=match[1])
     return {"cause": cause}
 
 
-@parser.add
+@parser._add
 def cannot_convert_dictionary_update_sequence(
-    message: str, _frame: FrameType, tb_data: TracebackData
+    message: str, tb_data: TracebackData
 ) -> CauseInfo:
     if "cannot convert dictionary update sequence element" not in message:
         return {}
@@ -983,12 +1017,12 @@ def cannot_convert_dictionary_update_sequence(
     return {"cause": cause, "suggest": hint}
 
 
-@parser.add
-def builtin_callable_has_no_len(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def builtin_callable_has_no_len(message: str, tb_data: TracebackData) -> CauseInfo:
     if message != "object of type 'builtin_function_or_method' has no len()":
         return {}
+
+    frame = tb_data.exception_frame
     all_objects = info_variables.get_all_objects(tb_data.bad_line, frame)["name, obj"]
     for name, obj in all_objects:
         if name == "len":
@@ -1007,12 +1041,12 @@ def builtin_callable_has_no_len(
     return {"cause": cause, "suggest": hint}
 
 
-@parser.add
-def function_has_no_len(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def function_has_no_len(message: str, tb_data: TracebackData) -> CauseInfo:
     if message != "object of type 'function' has no len()":
         return {}
+
+    frame = tb_data.exception_frame
     all_objects = info_variables.get_all_objects(tb_data.bad_line, frame)["name, obj"]
     for name, obj in all_objects:
         if name == "len":
@@ -1031,12 +1065,12 @@ def function_has_no_len(
     return {"cause": cause, "suggest": hint}
 
 
-@parser.add
-def vars_arg_must_have_dict(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def vars_arg_must_have_dict(message: str, tb_data: TracebackData) -> CauseInfo:
     if message != "vars() argument must have __dict__ attribute":
         return {}
+
+    frame = tb_data.exception_frame
     cause = _(
         "The function `vars` is used to list the content of the\n"
         "`__dict__` attribute of an object.\n"
@@ -1057,20 +1091,20 @@ def vars_arg_must_have_dict(
     return {"cause": cause}
 
 
-@parser.add
-def function_got_multiple_argument(
-    message: str, frame: FrameType, tb_data: TracebackData
-) -> CauseInfo:
+@parser._add
+def function_got_multiple_argument(message: str, tb_data: TracebackData) -> CauseInfo:
     pattern = r"(.*)\(\) got multiple values for argument '(.*)'"
     match = re.search(pattern, message)
     if not match:
         return {}
-    function_name = match.group(1)
+
+    frame = tb_data.exception_frame
+    function_name = match[1]
     # Annoyingly, Python 3.10 inserts <locals> as part of the name of functions
     # defined locally, which is what we often do in unit tests.
     if ".<locals>." in function_name:
         function_name = function_name.split(".<locals>.")[1]
-    argument = match.group(2)
+    argument = match[2]
     cause = _(
         "You have specified the value of argument `{argument}` more than once\n"
         "when calling the function named `{function}`.\n"
@@ -1100,3 +1134,34 @@ def function_got_multiple_argument(
             "This function has the following arguments:\n`{arguments}`\n"
         ).format(function=function_name, arguments=str(arguments)[1:-1])
     return {"cause": cause}
+
+
+@parser._add
+def generator_has_no_len(message: str, tb_data: TracebackData) -> CauseInfo:
+    if message != "object of type 'generator' has no len()":
+        return {}
+    cause = _(
+        "I am guessing that you were trying to count the number of elements\n"
+        "produced by a generator expression. You first need to capture them\n"
+        "in a list:\n\n"
+    )
+    tokens = token_utils.get_significant_tokens(tb_data.bad_line)
+    nb_open = sum(1 for tok in tokens if tok == "(")
+    nb_close = sum(1 for tok in tokens if tok == ")")
+    if (
+        nb_open == nb_close
+        and nb_open >= 1
+        and tokens[0] == "len"
+        and tokens[1] == "("
+        and tokens[-1] == ")"
+    ):
+        tokens[1].string = "(["
+        tokens[-1].string = "])"
+        new_line = token_utils.untokenize(tokens)
+    else:
+        new_line = "len([...])"
+
+    cause += _("    {new_line}\n").format(new_line=new_line)
+    hint = _("You likely need to build a list first.\n")
+
+    return {"cause": cause, "suggest": hint}
